package io.nisum.spark.consumer;

import io.nisum.spark.entity.ProductInfo;
import org.apache.spark.sql.*;
import org.apache.spark.sql.streaming.OutputMode;
import org.apache.spark.sql.streaming.StreamingQuery;
import org.apache.spark.sql.streaming.StreamingQueryException;
import org.apache.spark.sql.types.DataType;
import org.apache.spark.sql.types.DataTypes;
import org.apache.spark.sql.types.StructType;

import java.util.concurrent.TimeoutException;

public class StreamingConsumer {
    public static void main(String[] args) throws TimeoutException, StreamingQueryException {

        SparkSession spark = SparkSession.builder()
                                .master("local[*]")
                                .appName("Data processing")
                                .getOrCreate();

        Dataset<Row> rowDataset = spark.readStream()
                .format("kafka")
                .option("kafka.bootstrap.servers", "127.0.0.1:9092")
                .option("subscribe", "product-csv-topic-1")
                .option("kafka.consumer.group.id", "localStream")
                .option("failOnDataLoss", "false")
                .option("startingOffsets", "earliest")
                .load();

        rowDataset.printSchema();

        StructType schema = Encoders.bean(ProductInfo.class).schema();

        String startTime = "20-05-2023";
        String endTime = "25-05-2023";

        Dataset<Row> filteredSet = rowDataset
                .selectExpr("cast(value as string) as data")
                .select(functions.from_json(functions.col("data"), schema ).alias("jsonData") )
                .select("jsonData.*")
                .filter(functions.col("deliveryStatus").$eq$eq$eq("shipped") )
                .filter(functions.col("date").geq(startTime).and(functions.col("date").leq(endTime)));

        filteredSet.printSchema();

        StreamingQuery query = filteredSet
                .writeStream()
                .outputMode(OutputMode.Append())
                .format("csv")
                .option("checkpointLocation", "checkpointlocaion/streamingjob")
                .option("path", "src/data/result")
                .start();

        query.awaitTermination();
    }
}
